@@ -10,8 +10,8 @@ from sherlockpipe.scoring.QuorumSdeBorderCorrectedSignalSelector import QuorumSd
 from sherlockpipe.scoring.QuorumSnrBorderCorrectedSignalSelector import QuorumSnrBorderCorrectedSignalSelector
 from sherlockpipe.scoring.SdeBorderCorrectedSignalSelector import SdeBorderCorrectedSignalSelector
 from sherlockpipe.scoring.SnrBorderCorrectedSignalSelector import SnrBorderCorrectedSignalSelector
-from sherlockpipe.sherlock import Sherlock
-from sherlockpipe.sherlock_target import SherlockTarget
+from sherlockpipe.search.sherlock import Sherlock
+from sherlockpipe.search.sherlock_target import SherlockTarget
 
 
 class TestsSherlock(unittest.TestCase):
@@ -163,7 +163,7 @@ class TestsSherlock(unittest.TestCase):
                                      detrends_number=1, max_runs=1, oversampling=0.05)], True).run()
             run_dir = "TIC259377017_[5]"
             self.assertTrue(os.path.exists(run_dir))
-            self.assertTrue(os.path.exists(run_dir + '/Phase_detrend_period_TIC259377017_[5]_0.15_days.png'))
+            self.assertTrue(os.path.exists(run_dir + '/Phase_detrend_period_TIC259377017_[5]_8.50_days.png'))
         finally:
             self.__clean(run_dir)
 
@@ -171,7 +171,7 @@ class TestsSherlock(unittest.TestCase):
         run_dir = None
         try:
             Sherlock([SherlockTarget(MissionObjectInfo('all', "EPIC 249631677", cadence=1800, high_rms_enabled=True,
-                                                            auto_detrend_enabled=False),
+                                                       auto_detrend_enabled=False),
                                      detrends_number=1, max_runs=1, oversampling=0.05)], False).run()
             run_dir = "EPIC249631677_all"
             self.assertTrue(os.path.exists(run_dir))
